@@ -30,7 +30,7 @@ class ExtensibleValidator(object):
             self._additional_type = True
         else:
             if "$ref" in addlProp:
-                refs = builder.resolve_classes([addlProp])
+                typ = builder.resolve_type(addlProp["$ref"], name)
             else:
                 uri = "{0}/{1}_{2}".format(
                     name, "<additionalProperties>", "<anonymous>"
@@ -38,23 +38,23 @@ class ExtensibleValidator(object):
                 builder.resolved[uri] = builder.construct(
                     uri, addlProp, (cb.ProtocolBase,)
                 )
-                refs = [builder.resolved[uri]]
+                typ = builder.resolved[uri]
 
-            self._additional_type = refs[0]
+            self._additional_type = typ
 
         for pattern, typedef in six.iteritems(schemadef.get("patternProperties", {})):
             if "$ref" in typedef:
-                refs = builder.resolve_classes([typedef])
+                typ = builder.resolve_type(typedef["$ref"], name)
             else:
                 uri = "{0}/{1}_{2}".format(name, "<patternProperties>", pattern)
 
                 builder.resolved[uri] = builder.construct(
                     uri, typedef, (cb.ProtocolBase,)
                 )
-                refs = [builder.resolved[uri]]
+                typ = builder.resolved[uri]
 
             self._pattern_types.append(
-                PatternDef(pattern=re.compile(pattern), schema_type=refs[0])
+                PatternDef(pattern=re.compile(pattern), schema_type=typ)
             )
 
     def _make_type(self, typ, val):
@@ -71,6 +71,14 @@ class ExtensibleValidator(object):
         ):
             return typ(val)
 
+        if isinstance(typ, cb.TypeProxy):
+            val = util.coerce_for_expansion(val)
+            if isinstance(val, dict):
+                val = typ(**val)
+            else:
+                val = typ(val)
+            return val
+
         raise validators.ValidationError(
             "additionalProperty type {0} was neither a literal "
             "nor a schema wrapper: {1}".format(typ, val)
@@ -87,7 +95,6 @@ class ExtensibleValidator(object):
                 return self._make_type(p.schema_type, val)
 
         if self._additional_type is True:
-
             valtype = [
                 k
                 for k, t in validators.SCHEMA_TYPE_MAPPING
@@ -96,7 +103,7 @@ class ExtensibleValidator(object):
             valtype = valtype[0]
             return MakeLiteral(name, valtype, val)
 
-        elif isinstance(self._additional_type, type):
+        elif isinstance(self._additional_type, (type, cb.TypeProxy)):
             return self._make_type(self._additional_type, val)
 
         raise validators.ValidationError(
